//@author: ain
//@help: creates layers of colors from 1D-texture
//@tags: color layers material
//@credits: based on wood effect by vux from directx 9 sdk

// transformations
float4x4 World : WORLD;
float4x3 WorldView  : WORLDVIEW;
float4x4 Projection : PROJECTION;


// light direction (view space)
float3 LightDir < string uiname = "Light Direction"; > = {0.577, -0.577, 0.577};

// light colors
float4 Ambient : COLOR < string uiname = "Ambient Color"; > = { 0.3f, 0.3f, 0.3f, 1.0f };   
float4 Diffuse : COLOR < string uiname = "Diffuse Color"; > = { 1.0f, 1.0f, 1.0f, 1.0f };   
float4 Specular : COLOR < string uiname = "Specular Color"; > = { 0.6f, 0.6f, 0.6f, 1.0f };   

float  Power < string uiname = "Power"; >   = 64.0f;


// scale of the layers /inversely/
float Scale < string uiname= "Slice Scale"; > = 10.0f;
// smoothing /inversely/
int Smooth  < string uiname = "Smoothing Amount"; >   = 10;
// phase
float Phase = 0;

// texture
texture ColorTex < string uiname= "Color Texture"; >;
int TexWidth < string uiname= "Texture Width"; > = 12;
sampler Samp = sampler_state
{
    texture = <ColorTex>;
    AddressU  = WRAP;
};


// vs to ps structure
struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float3 Diff : COLOR0;
    float3 Spec : COLOR1;
    float3 PixPos : TEXCOORD0;
    float ViewZ: TEXCOORD1;
};

// VERTEXSHADER
VS_OUTPUT VS(
    float3 Pos  : POSITION,
    float3 Norm : NORMAL)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    float3 L = -LightDir;
    float3 P = mul(float4(Pos, 1), (float4x3)WorldView);    // position (view space)
    float3 N = normalize(mul(Norm, (float3x3)WorldView));   // normal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);            // reflection vector (view space)
    float3 V = -normalize(P);                               // view direction (view space)

    Out.Pos  = mul(float4(P, 1), Projection);             // position (projected)
    Out.Diff = Ambient + Diffuse * max(0, dot(N, L)); 
    Out.Spec = Specular * pow(max(0, dot(R, V)), Power/4);
    
    Out.PixPos = mul(float4(Pos, 1), (float4x3)World);     // position (World space)
    
    return Out;
}

// PIXELSHADER
float4 SPHERE(VS_OUTPUT In) : COLOR
{
    float4 Color = {0, 0, 0, 0};
    float4 SelfColor;
    float4 NextColor;
    float4 TexColor;
    float3 PP = In.PixPos;
    PP = length(PP.xyz); //tweak here to get another layer shape
    float PixelWidth = 1.0 / TexWidth;
    float TexCd;
    TexCd = Scale * PP + Phase * TexWidth; 
    float PosOnLine = frac(TexCd);
    float TexCdNext = (floor(TexCd) + 1) * PixelWidth + (PixelWidth * 0.5);
    TexCd = floor(TexCd) * PixelWidth + (PixelWidth * 0.5);
    SelfColor = tex1D(Samp, TexCd);
    NextColor = tex1D(Samp, TexCdNext);
    TexColor = lerp(NextColor, SelfColor, saturate((1 - PosOnLine) * Smooth));
    Color.rgb = In.Diff * TexColor.rgb + In.Spec;
    Color.a = 1;
    return Color;
}



//TECHNIQUE
technique TCandyLayers
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 SPHERE();
    }
}
